"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.default = void 0;

var _store = _interopRequireDefault(require("../../store"));

var _transformFields = require("./transform-fields");

var _isExcluded = require("../ingest-remote-schema/is-excluded");

var _helpers = require("./helpers");

const unionType = typeBuilderApi => {
  const {
    schema,
    type,
    pluginOptions
  } = typeBuilderApi;
  const types = type.possibleTypes.filter(possibleType => !(0, _isExcluded.typeIsExcluded)({
    pluginOptions,
    typeName: possibleType.name
  })).map(possibleType => (0, _helpers.buildTypeName)(possibleType.name));

  if (!types || !types.length) {
    return false;
  }

  let unionType = {
    name: (0, _helpers.buildTypeName)(type.name),
    types,
    resolveType: node => {
      if (node.__typename) {
        return (0, _helpers.buildTypeName)(node.__typename);
      }

      return null;
    },
    extensions: {
      infer: false
    }
  }; // @todo add this as a plugin option

  unionType = (0, _helpers.filterTypeDefinition)(unionType, typeBuilderApi, `UNION`);
  return schema.buildUnionType(unionType);
};

const interfaceType = typeBuilderApi => {
  const {
    type,
    schema
  } = typeBuilderApi;

  const state = _store.default.getState();

  const {
    ingestibles
  } = state.remoteSchema;
  const {
    nodeInterfaceTypes
  } = ingestibles;
  const implementingTypes = (0, _helpers.getTypesThatImplementInterfaceType)(type);
  const transformedFields = (0, _transformFields.transformFields)({
    parentInterfacesImplementingTypes: implementingTypes,
    parentType: type,
    fields: type.fields
  });

  if (!transformedFields) {
    return null;
  }

  let typeDef = {
    name: (0, _helpers.buildTypeName)(type.name),
    fields: transformedFields,
    extensions: {
      infer: false
    }
  }; // if this is a node interface type

  if (nodeInterfaceTypes.includes(type.name)) {
    // we add nodeType (post type) to all nodes as they're fetched
    // so we can add them to node interfaces as well in order to filter
    // by a couple different content types
    typeDef.fields[`nodeType`] = `String`;
    typeDef.interfaces = [`Node`];
  } else {
    // otherwise this is a regular interface type so we need to resolve the type name
    typeDef.resolveType = node => node !== null && node !== void 0 && node.__typename ? (0, _helpers.buildTypeName)(node.__typename) : null;
  } // @todo add this as a plugin option


  typeDef = (0, _helpers.filterTypeDefinition)(typeDef, typeBuilderApi, `INTERFACE`);
  return schema.buildInterfaceType(typeDef);
};

const objectType = typeBuilderApi => {
  var _type$interfaces;

  const {
    type,
    gatsbyNodeTypes,
    fieldAliases,
    fieldBlacklist,
    schema,
    isAGatsbyNode
  } = typeBuilderApi;
  const transformedFields = (0, _transformFields.transformFields)({
    fields: type.fields,
    parentType: type,
    gatsbyNodeTypes,
    fieldAliases,
    fieldBlacklist
  }); // if all child fields are excluded, this type shouldn't exist.
  // check null first, otherwise cause:
  // TypeError: Cannot convert undefined or null to object at Function.keys (<anonymous>)
  // Also cause wordpress blog site build failure in createSchemaCustomization step

  if (!transformedFields || !Object.keys(transformedFields).length) {
    return false;
  }

  let objectType = {
    name: (0, _helpers.buildTypeName)(type.name),
    fields: transformedFields,
    description: type.description,
    extensions: {
      infer: false
    }
  };

  if (type.interfaces) {
    objectType.interfaces = type.interfaces.filter(interfaceType => {
      const interfaceTypeSettings = (0, _helpers.getTypeSettingsByType)(interfaceType);
      return !interfaceTypeSettings.exclude && (0, _helpers.fieldOfTypeWasFetched)(type) && (0, _helpers.fieldOfTypeWasFetched)(interfaceType);
    }).map(({
      name
    }) => (0, _helpers.buildTypeName)(name));
  }

  if (gatsbyNodeTypes.includes(type.name) || isAGatsbyNode || // this accounts for Node types that weren't fetched because
  // they have no root field to fetch a single node of this type
  // removing them from the schema breaks the build though
  // @todo instead, if a node type isn't fetched, remove it
  // from the entire schema
  type !== null && type !== void 0 && (_type$interfaces = type.interfaces) !== null && _type$interfaces !== void 0 && _type$interfaces.find(({
    name
  }) => name === `Node`)) {
    // this is used to filter the node interfaces
    // by different content types (post types)
    objectType.fields[`nodeType`] = `String`;
    objectType.interfaces = [`Node`, ...objectType.interfaces];
  } // @todo add this as a plugin option


  objectType = (0, _helpers.filterTypeDefinition)(objectType, typeBuilderApi, `OBJECT`);
  return schema.buildObjectType(objectType);
};

const enumType = ({
  schema,
  type
}) => schema.buildEnumType({
  name: (0, _helpers.buildTypeName)(type.name),
  values: type.enumValues.reduce((accumulator, {
    name
  }) => {
    accumulator[name] = {
      name
    };
    return accumulator;
  }, {}),
  description: type.description
});

var _default = {
  unionType,
  interfaceType,
  objectType,
  enumType
};
exports.default = _default;
//# sourceMappingURL=build-types.js.map